import os

from decouple import config

# Constants
API_KEY_HEADER = 'X-API-KEY'

# Configs
ACCOUNT = config('DBLUE_MLWATCH_ACCOUNT', default=None)
PROJECT = config('DBLUE_MLWATCH_PROJECT', default=None)
API_KEY = config('DBLUE_MLWATCH_API_KEY', default=None)
ENDPOINT = config('DBLUE_MLWATCH_ENDPOINT', default='https://us-central1-dblue-dev-235513.cloudfunctions.net/collect')
METRICS_TRACKING_INTERVAL = config('DBLUE_MLWATCH_METRICS_TRACKING_INTERVAL', default=5, cast=int)

LOG_LEVEL = config("LOG_LEVEL", default="INFO")

PROJECT_ROOT = os.path.abspath(os.path.dirname(__file__))



import psutil


def get_network_usage():
    # get IO statistics since boot
    net_io = psutil.net_io_counters()

    usage = {
        'bytes_sent': net_io.bytes_sent,
        'bytes_received': net_io.bytes_recv,
        'packets_sent': net_io.packets_sent,
        'packets_received': net_io.packets_recv,
        'error_in': net_io.errin,
        'error_out': net_io.errout,
        'drop_in': net_io.dropin,
        'drop_out': net_io.dropout,
    }

    return usage



import shutil

import psutil


def get_disk_partition_usage():
    # get all disk partitions
    disk_partitions = psutil.disk_partitions()
    partitions = {}

    for partition in disk_partitions:
        info = {
            'name': partition.device,
            'mount_point': partition.mountpoint,
            'fs_type': partition.fstype,
        }

        try:
            partition_usage = psutil.disk_usage(partition.mountpoint)
        except PermissionError:
            # this can happen when disk is not ready
            partitions[partition.device] = info
            continue

        info['size'] = partition_usage.total
        info['used'] = partition_usage.used
        info['free'] = partition_usage.free
        info['used_percent'] = partition_usage.percent

        partitions[partition.device] = info

    return partitions


def get_disk_usage():
    # get IO statistics since boot
    disk_io = psutil.disk_io_counters()
    total, used, free = shutil.disk_usage("/")

    usage = {
        'total': total,
        'used': used,
        'free': free,
        'read_count': disk_io.read_count,
        'write_count': disk_io.write_count,
        'read_bytes': disk_io.read_bytes,
        'write_bytes': disk_io.write_bytes,
    }

    return usage



import os
import platform

from distutils import spawn
from subprocess import PIPE, Popen

from dblue_mlwatch.logger import logger


def safe_float_cast(num_str):
    try:
        number = float(num_str)
    except ValueError:
        number = float('nan')
    return number


def get_gpu_usage():
    if platform.system() == "Windows":
        # If the platform is Windows and nvidia-smi 
        # could not be found from the environment path, 
        # try to find it from system drive with default installation path
        nvidia_smi = spawn.find_executable('nvidia-smi')
        if nvidia_smi is None:
            nvidia_smi = "%s\\Program Files\\NVIDIA Corporation\\NVSMI\\nvidia-smi.exe" % os.environ['systemdrive']
    else:
        nvidia_smi = "nvidia-smi"

    # Get ID, processing and memory utilization for all GPUs

    query_params = ",".join([
        'index',
        'uuid',
        'utilization.gpu',
        'memory.total',
        'memory.used',
        'memory.free',
        'driver_version',
        'name',
        'gpu_serial',
        'display_active',
        'display_mode',
        'temperature.gpu',
    ])

    try:
        p = Popen([
            nvidia_smi,
            "--query-gpu={}".format(query_params),
            "--format=csv,noheader,nounits"
        ], stdout=PIPE
        )
        stdout, _ = p.communicate()
    except Exception as e:
        logger.exception(e)
        return {}

    output = stdout.decode('UTF-8')

    lines = output.split(os.linesep)

    num_devices = len(lines) - 1
    gpus = {}

    for g in range(num_devices):
        line = lines[g]
        values = line.split(', ')

        info = {
            'device_id': int(values[0]),
            'uuid': values[1],
            'utilization': safe_float_cast(values[2]) / 100,
            'memory_total': safe_float_cast(values[3]),
            'memory_used': safe_float_cast(values[4]),
            'memory_free': safe_float_cast(values[5]),
            'driver': values[6],
            'name': values[7],
            'temp': safe_float_cast(values[11]),

        }

        gpus['device_id'] = info
    return gpus



from dblue_mlwatch import config
from dblue_mlwatch.constants import EventType
from dblue_mlwatch.core.func_thread import FuncThread
from dblue_mlwatch.logger import logger
from dblue_mlwatch.trackers.base import BaseTracker


class MetricsTracker(BaseTracker):

    def __init__(self, *args, **kwargs):
        super(MetricsTracker, self).__init__(*args, **kwargs)

        self.thread = None

    def track_system_metrics(self, interval=config.METRICS_TRACKING_INTERVAL):
        self.track_system_info()
        self.track_system_usage(interval=interval)

    def track_system_info(self) -> bool:
        """
        Track system info such as operating system and hardware details
        :return: Emit status
        """

        from dblue_mlwatch import system

        event = {
            'event_type': EventType.SYSTEM_INFO,
            'os': system.get_os_info(),
            'hardware': system.get_hardware_info(),
        }

        statuses = self.emit(events=[event])
        return statuses[0]

    def _track_system_usage(self) -> bool:
        """
        Emit system usage event
        :return: Emit status
        """

        from dblue_mlwatch import system

        event = {
            'event_type': EventType.SYSTEM_USAGE,
            'cpu': system.get_cpu_usage(),
            'memory': system.get_memory_usage(),
            'disk': system.get_disk_usage(),
            'network': system.get_network_usage(),
            'gpu': system.get_gpu_usage(),
        }

        statuses = self.emit(events=[event])
        return statuses[0]

    def track_system_usage(self, interval):
        """
        Track system usage at interval
        :return:
        """
        try:
            self.thread = FuncThread(delay=interval, func=self._track_system_usage)
        except Exception as e:
            logger.exception(e)

    def stop(self):
        """
        Stop metrics monitoring thread
        :return:
        """
        if self.thread:
            self.thread.stop()



import json
import sys
import traceback
import uuid

from typing import Dict, List

from dblue_mlwatch.constants import EventType, Types
from dblue_mlwatch.exceptions import DblueMLWatchException
from dblue_mlwatch.logger import logger
from dblue_mlwatch.trackers.base import BaseTracker
from dblue_mlwatch.validation import validate_schema
from dblue_mlwatch.validation.schemas import PREDICTION_SCHEMA


class PredictionTracker(BaseTracker):

    def __init__(self, *args, **kwargs):
        super(PredictionTracker, self).__init__(*args, **kwargs)

    def _create_prediction_event(
            self,
            features: Types.Features,
            prediction: Types.Prediction,
            prediction_probs: Types.PredictionProbs = None,
            unique_id: Types.UniqueId = None
    ) -> Types.PredictionEvent:
        """
        Create prediction event dict
        :param features: Model features
        :param prediction: Prediction from model
        :param prediction_probs: Prediction probability for classification models
        :param unique_id: Unique id for the model request, used for combining feedback at later stage
        :return: Dict of the prediction event
        """

        unique_id = unique_id or uuid.uuid4().hex
        _features = None
        _prediction_probs = None

        try:
            _features = json.dumps(features) if features else None
            _prediction_probs = json.dumps(prediction_probs) if prediction_probs else None
        except Exception as e:
            raise DblueMLWatchException("Failed to serialize prediction data: %s" % e)

        event = {
            'event_type': EventType.PREDICTION,
            'features': _features,
            'prediction': prediction,
            'prediction_probs': _prediction_probs,
            'unique_id': unique_id,
        }

        return unique_id, event

    def capture_prediction(self, event: Dict) -> Types.CaptureStatus:
        """
        Capture prediction event from dict
        :param event: Prediction event in dictionary format
        :return: Capture status
        """

        is_valid, errors = validate_schema(data=event, schema=PREDICTION_SCHEMA)

        if not is_valid:
            raise DblueMLWatchException("Prediction data is not as per the schema: %s" % errors)

        unique_id, event = self._create_prediction_event(
            features=event.get('features'),
            prediction=event.get('prediction'),
            prediction_probs=event.get('prediction_probs'),
            unique_id=event.get("unique_id")
        )

        statuses = self.emit(events=[event])

        return unique_id, statuses[0]

    def capture_prediction_batch(self, events: List[Dict]) -> List[Types.CaptureStatus]:
        """
        Capture prediction events in batch
        :param events: List of events
        :return: List of capture status
        """

        _events = []
        _unique_ids = []

        for event in events:

            is_valid, errors = validate_schema(data=event, schema=PREDICTION_SCHEMA)

            if not is_valid:
                raise DblueMLWatchException("Prediction data is not as per the schema: %s" % errors)

            unique_id, event = self._create_prediction_event(
                features=event.get('features'),
                prediction=event.get('prediction'),
                prediction_probs=event.get('prediction_probs'),
                unique_id=event.get("unique_id")
            )

            _unique_ids.append(unique_id)
            _events.append(event)

        statuses = self.emit(events=_events)
        return list(zip(_unique_ids, statuses))

    def capture_exception(self, message: str = None) -> bool:
        """
        Capture exceptions
        :param message: Error message
        :return: Emit status
        """
        exc_info = sys.exc_info()

        event = {
            'event_type': EventType.EXCEPTION,
            'message': message,
            'exec_info': traceback.format_exception(*exc_info),
        }

        logger.exception(msg=message)

        statuses = self.emit(events=[event])
        return statuses[0]

    def capture_request_log(self, request: Dict, response: Dict, response_time: float) -> bool:
        """
        Capture request logs with response and time taken to serve the request

        :param request: Request parameters
        :param response:  Response
        :param response_time: Time taken to serve the request
        :return: Capture status
        """

        _request = None
        _response = None

        try:
            _request = json.dumps(request) if request else None
            _response = json.dumps(response) if response else None

        except Exception as e:
            raise DblueMLWatchException("Failed to serialize prediction data: %s" % e)

        event = {
            'event_type': EventType.REQUEST_LOG,
            'request': _request,
            'response': _response,
            'response_time': response_time,
        }

        statuses = self.emit(events=[event])
        return statuses[0]



def get_size(bytes_, suffix="B"):
    """
    Scale bytes to its proper format
    e.g:
        1253656 => '1.20MB'
        1253656678 => '1.17GB'
    """
    factor = 1024
    value = ''
    for unit in ["", "K", "M", "G", "T", "P"]:
        if bytes_ < factor:
            value = f"{bytes_:.2f}{unit}{suffix}"
            break
        bytes_ /= factor

    return value



import platform
import time

from datetime import datetime
from typing import Dict, List

from dblue_mlwatch import config
from dblue_mlwatch.exceptions import DblueMLWatchException
from dblue_mlwatch.stores.rest import RestStore
from dblue_mlwatch.version import VERSION


class BaseTracker:

    def __init__(self, model_id: str, model_version: str, account: str = None, project: str = None,
                 api_key: str = None):
        """
        :param model_id: Dblue MLWatch model id
        :param model_version: Dblue MLWatch model version
        :param account: Dblue MLWatch account id
        :param project: Dblue MLWatch project id
        :param api_key: Dblue MLWatch API Key
        """

        model_id = model_id
        model_version = model_version

        account = account or config.ACCOUNT
        project = project or config.PROJECT
        api_key = api_key or config.API_KEY

        if not account:
            raise DblueMLWatchException(
                "Please specify account in the func call or in DBLUE_MLWATCH_ACCOUNT env variable"
            )

        if not project:
            raise DblueMLWatchException(
                "Please specify project in the func call or in DBLUE_MLWATCH_PROJECT env variable"
            )

        if not api_key:
            raise DblueMLWatchException(
                "Please specify api_key in the function call or in DBLUE_MLWATCH_API_KEY env variable"
            )

        self.account = account
        self.project = project
        self.model_id = model_id
        self.model_version = model_version
        self.api_key = api_key
        self.endpoint = config.ENDPOINT

        self.store = RestStore(endpoint=self.endpoint, api_key=self.api_key)

    def emit(self, events: List[Dict]) -> List:
        """
        Emit events to store
        :param events: List of events
        :return: List of emit statuses
        """
        common = {
            'account': self.account,
            'project': self.project,
            'model_id': self.model_id,
            'model_version': self.model_version,
            'sdk': 'py-{}'.format(VERSION),
            'local_time': datetime.now().isoformat(),
            'utc_time': datetime.utcnow().isoformat(),
            'event_tstamp': time.time(),
            'tz_offset': time.strftime("%z", time.gmtime()),
            'tz_name': time.tzname[0],
            'hostname': platform.node(),
        }

        for event in events:
            event.update(common)

        return self.store.send(events=events)



PREDICTION_SCHEMA = {
    'features': {
        'type': 'dict',
        'required': True,
    },
    'prediction': {
        'type': ['number', 'boolean', 'string'],
        'required': True,
    },
    'prediction_probs': {
        'type': 'dict',
        'nullable': True,
    },
    'unique_id': {
        'type': 'string',
        'nullable': True
    },
}



import time

from threading import Thread

from dblue_mlwatch.logger import logger


class FuncThread(Thread):
    def __init__(self, delay, func, *args, **kwargs):
        super(FuncThread, self).__init__()
        self.stopped = False
        self.delay = delay  # Time between calls
        self.func = func
        self.args = args
        self.kwargs = kwargs

        self.start()

    def run(self):
        while not self.stopped:
            # Call the function
            try:
                self.func(*self.args, **self.kwargs)
            except Exception as e:
                logger.exception(e)

            time.sleep(self.delay)

    def stop(self):
        self.stopped = True



import psutil


def get_memory_usage():
    memory = psutil.virtual_memory()
    swap = psutil.swap_memory()

    return {
        'total': memory.total,
        'available': memory.available,
        'used': memory.used,
        'used_percent': memory.percent,
        'swap_total': swap.total,
        'swap_free': swap.free,
        'swap_used': swap.used,
        'swap_used_percent': swap.percent,
    }



import platform
import re
import uuid

import psutil

from dblue_mlwatch.logger import logger
from dblue_mlwatch.utils import get_size


def get_os_info():
    try:
        info = {
            'platform': platform.platform(),
            'system': platform.system(),
            'release': platform.release(),
            'version': platform.version(),
        }
        return info
    except Exception:
        logger.exception("Failed to read operating system information")


def get_hardware_info():
    try:
        memory = psutil.virtual_memory()

        info = {
            'architecture': platform.machine(),
            'processor': platform.processor(),
            'mac_address': ':'.join(re.findall('..', '%012x' % uuid.getnode())),
            'cores': psutil.cpu_count(),
            'memory': get_size(memory.total),
        }

        return info
    except Exception:
        logger.exception("Failed to read hardware information")



from typing import Dict, List

from dblue_mlwatch.constants import EventType, Types
from dblue_mlwatch.exceptions import DblueMLWatchException
from dblue_mlwatch.trackers.base import BaseTracker
from dblue_mlwatch.validation import validate_schema
from dblue_mlwatch.validation.schemas import FEEDBACK_SCHEMA


class FeedbackTracker(BaseTracker):

    def __init__(self, *args, **kwargs):
        super(FeedbackTracker, self).__init__(*args, **kwargs)

    def _create_feedback_event(self, unique_id: str, target: Types.Target) -> Dict:
        """
        Create feedback event dict
        :param unique_id: Unique id for combining the feedback with prediction event
        :param target: Target label for the event
        :return: Event dict
        """

        return {
            'event_type': EventType.FEEDBACK,
            'target': target,
            'unique_id': unique_id,
        }

    def capture_feedback(self, event: Dict) -> Types.CaptureStatus:
        """
        Capture feedback event from dictionary
        :param event: Feedback event in dict format
        :return: Capture status
        """

        is_valid, errors = validate_schema(data=event, schema=FEEDBACK_SCHEMA)

        if not is_valid:
            raise DblueMLWatchException("Feedback data is not as per the schema: %s" % errors)

        unique_id = event.get('unique_id')

        event = self._create_feedback_event(
            unique_id=unique_id,
            target=event.get('target')
        )

        statuses = self.emit(events=[event])

        return unique_id, statuses[0]

    def capture_feedback_batch(self, events: List[Dict]) -> List[Types.CaptureStatus]:
        """
        Capture feedback event in batch from dictionary
        :param events: List of feedback events
        :return: Capture statuses
        """

        _events = []
        _unique_ids = []

        for event in events:
            is_valid, errors = validate_schema(data=event, schema=FEEDBACK_SCHEMA)

            if not is_valid:
                raise DblueMLWatchException("Feedback data is not as per the schema: %s" % errors)

            unique_id = event.get('unique_id')

            _unique_ids.append(unique_id)

            event = self._create_feedback_event(
                unique_id=unique_id,
                target=event.get('target')
            )

            _events.append(event)

        statuses = self.emit(events=events)
        return list(zip(_unique_ids, statuses))



class BaseStore:

    def __init__(self, **kwargs):
        pass

    def send(self, events: list, **kwargs) -> list:
        """
        Send events to store
        :param events: List of records
        :param kwargs:
        :return: List of status(Boolean) suggesting if the event successfully sent to store or not
        """
        raise NotImplementedError()



from typing import Dict, Tuple

from cerberus import Validator


def validate_schema(data, schema) -> Tuple[bool, Dict]:
    validator = Validator(schema)
    status = validator.validate(data)

    return status, validator.errors



from typing import Dict, Tuple, Union


class Types:
    Features = Dict
    Prediction = Union[str, int, float, bool]
    PredictionProbs = Union[Dict, None]
    UniqueId = Union[str, None]
    CaptureStatus = Tuple[str, bool]
    PredictionEvent = Tuple[str, dict]
    Target = Union[str, int, float, bool]



import asyncio
import json
import socket
import time

import aiohttp

from tqdm import tqdm

from dblue_mlwatch.config import API_KEY_HEADER
from dblue_mlwatch.logger import logger
from dblue_mlwatch.stores.base import BaseStore

DEFAULT_HEADERS = {
    "Content-Type": "application/json"
}


class RestStore(BaseStore):
    def __init__(self, endpoint, api_key, **kwargs):
        super(RestStore, self).__init__(**kwargs)

        self.url = endpoint
        self.api_key = api_key

        self.headers = DEFAULT_HEADERS
        self.headers[API_KEY_HEADER] = api_key

    def send(self, events: list, **kwargs) -> list:
        """
        Send events to the collector endpoint
        :param events: List of events to send
        :param kwargs:
        :return: List of status(Boolean) suggesting if the event successfully sent to collector or not
        """

        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
        statuses = loop.run_until_complete(self._send(events=events))

        return statuses

    async def _send(self, events) -> list:
        statuses = []
        connector = aiohttp.TCPConnector(family=socket.AF_INET, ttl_dns_cache=300)

        async with aiohttp.ClientSession(connector=connector) as session:
            # Only use tqdm for batch sending
            events = tqdm(events) if len(events) > 1 else events

            for event in events:
                status_code, response = await self._make_request(session=session, data=event, headers=self.headers)

                if status_code == 200:
                    statuses.append(True)
                else:
                    statuses.append(False)
                    logger.debug("Failed request: %s", event)
                    logger.error("Request failed with status code %s: %s", status_code, response)

        return statuses

    async def _make_request(self, session, data, retries=3, retry_interval=3, **kwargs) -> (int, str):

        for i in range(retries):
            logger.debug("Sending request: %s", json.dumps(data))
            async with session.post(url=self.url, json=data, **kwargs) as response:
                status_code = response.status
                response = await response.text()

            if status_code == 200:
                return status_code, response

            logger.error(
                "Request failed with status code %s, retrying up to %s more times: %s",
                status_code,
                retries - i - 1,
                response
            )
            time.sleep(retry_interval)
        logger.error("Request failed to return code 200 after %s tries", retries)
        return status_code, response



#!/usr/bin/env python

import os
import sys

from setuptools import find_packages, setup
from setuptools.command.test import test as TestCommand


def version():
    project_root = os.path.abspath(os.path.dirname(__file__))

    version_file = os.path.join(project_root, "dblue_mlwatch", "configs", "version.txt")

    with open(version_file, 'r') as f:
        return f.read().strip()


def requirements():
    with open('requirements/requirements-base.txt') as f:
        return f.readlines()


def read_readme():
    with open('README.md') as f:
        return f.read()


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []  # pylint: disable=attribute-defined-outside-init
        self.test_suite = True  # pylint: disable=attribute-defined-outside-init

    def run_tests(self):
        import pytest  # pylint: disable=import-outside-toplevel
        errcode = pytest.main(self.test_args)
        sys.exit(errcode)


PACKAGE_NAME = 'dblue-mlwatch'
PACKAGE_VERSION = version()

setup(
    name=PACKAGE_NAME,
    version=PACKAGE_VERSION,
    description='Dblue MLWatch SDK for real-time machine learning model monitoring',
    long_description=read_readme(),
    long_description_content_type="text/markdown",
    maintainer='Rajesh Hegde',
    maintainer_email='rh@dblue.ai',
    author='Rajesh Hegde',
    author_email='rh@dblue.ai',
    url='https://github.com/dblueai/dblue-mlwatch-python-sdk',
    license='Apache2',
    platforms='any',
    packages=find_packages(),
    package_dir={'dblue_mlwatch': 'dblue_mlwatch'},
    package_data={
        'dblue_mlwatch': ['configs/*.yaml', 'configs/*.txt']
    },
    keywords=[
        'dblue',
        'machine-learning',
        'deep-learning',
        'monitoring',
        'mlwatch',

    ],
    install_requires=requirements(),
    extras_require={
        "system-metrics": [
            "psutil==5.7.0",
        ],
    },
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Topic :: Scientific/Engineering :: Artificial Intelligence'
    ],
    tests_require=[
        "pytest",
    ],
    cmdclass={'test': PyTest}
)



from dblue_mlwatch import config
from dblue_mlwatch.logger import logger
from dblue_mlwatch.trackers.feedback import FeedbackTracker
from dblue_mlwatch.trackers.metrics import MetricsTracker
from dblue_mlwatch.trackers.prediction import PredictionTracker


class MLWatch:
    def __init__(self, model_id, model_version, account=None, project=None, api_key=None):
        """
        Monitor prediction, feedback, request logs etc
        :param model_id: Dblue MLWatch model id
        :param model_version: Dblue MLWatch model version
        :param account: Dblue MLWatch account id
        :param project: Dblue MLWatch project id
        :param api_key: Dblue MLWatch API Key
        """

        self.model_id = model_id
        self.model_version = model_version

        self.account = account or config.ACCOUNT
        self.project = project or config.PROJECT
        self.api_key = api_key or config.API_KEY

        self.p_tracker = PredictionTracker(
            model_id=model_id, model_version=model_version, account=account, project=project, api_key=api_key
        )
        self.f_tracker = FeedbackTracker(
            model_id=model_id, model_version=model_version, account=account, project=project, api_key=api_key
        )
        self.m_tracker = MetricsTracker(
            model_id=model_id, model_version=model_version, account=account, project=project, api_key=api_key
        )

    def capture_prediction(self, data):
        """
        Capture prediction data from dict
        :param data: Prediction dat in dictionary format
        :return: Capture status
        """

        try:
            return self.p_tracker.capture_prediction(event=data)
        except Exception:
            self.p_tracker.capture_exception("Failed to capture prediction")

        return None

    def capture_prediction_batch(self, data):
        """
        Capture prediction data in batch
        :param data: List of data
        :return: List of capture status
        """

        try:
            return self.p_tracker.capture_prediction_batch(events=data)
        except Exception:
            self.p_tracker.capture_exception("Failed to capture prediction in batch")

        return None

    def capture_exception(self, message=None):
        """
        Capture exceptions
        :param message: Error message
        :return: Emit status
        """

        try:
            return self.p_tracker.capture_exception(message=message)
        except Exception:
            logger.exception("Failed to capture exception")

        return None

    def capture_request_log(self, request, response, response_time):
        """
        Capture request logs with response and time taken to serve the request

        :param request: Request parameters
        :param response:  Response
        :param response_time: Time taken to serve the request
        :return: Capture status
        """

        try:
            return self.p_tracker.capture_request_log(request=request, response=response, response_time=response_time)
        except Exception:
            self.p_tracker.capture_exception("Failed to capture request log")

        return None

    def capture_feedback(self, data):
        """
        Capture feedback data from dictionary
        :param data: Feedback data in dict format
        :return: Capture status
        """

        try:
            return self.f_tracker.capture_feedback(event=data)
        except Exception:
            self.p_tracker.capture_exception("Failed to capture feedback")

        return None

    def capture_feedback_batch(self, data):
        """
        Capture feedback data in batch from dictionary
        :param data: List of feedback data
        :return: Capture statuses
        """

        try:
            return self.f_tracker.capture_feedback_batch(events=data)
        except Exception:
            self.p_tracker.capture_exception("Failed to capture feedback in batch")

        return None

    def track_system_metrics(self, interval: int = config.METRICS_TRACKING_INTERVAL):
        try:
            import psutil  # noqa

            self.m_tracker.track_system_metrics(interval=interval)
        except ImportError:
            logger.exception(
                "System metrics module is not installed."
                "Please install Dblue MLWatch with pip install dblue_mlwatch[system-metrics]"
            )
        except Exception:
            self.p_tracker.capture_exception("Failed to track system metrics")

    def stop_tracking_system_metrics(self):
        self.m_tracker.stop()



import logging.config
import os

import yaml

from dblue_mlwatch.config import LOG_LEVEL, PROJECT_ROOT

logger_config_file_path = os.path.join(PROJECT_ROOT, "configs", "logger.yaml")

with open(logger_config_file_path) as f:
    logging_config = yaml.safe_load(f.read())
    logging.config.dictConfig(logging_config)

logger = logging.getLogger('dblue_mlwatch')
logger.setLevel(level=LOG_LEVEL)



import psutil

from dblue_mlwatch.logger import logger


def get_cpu_load(interval=1):
    try:
        cpu_loads = [x / psutil.cpu_count() * 100 for x in psutil.getloadavg()]

        if interval == 1:
            return cpu_loads[0]

        if interval == 5:
            return cpu_loads[1]

        if interval == 15:
            return cpu_loads[2]

        return 0
    except Exception as e:
        logger.exception(e)
        return 0


def get_cpu_usage():
    return {
        'load': get_cpu_load(),
        'usage_percent': psutil.cpu_percent(),
    }
